@@ -1,17 +1,24 @@
 """AwesomeVersion."""
 from types import TracebackType
-from typing import Any, List, Optional, Type, Union
+from typing import Dict, List, Optional, Type, Union
 
-from .exceptions import AwesomeVersionCompare, AwesomeVersionStrategyException
+from .exceptions import AwesomeVersionCompareException, AwesomeVersionStrategyException
 from .handlers import CompareHandlers
-from .strategy import VERSION_STRATEGIES, AwesomeVersionStrategy
+from .strategy import (
+    VERSION_STRATEGIES,
+    VERSION_STRATEGIES_DICT,
+    AwesomeVersionStrategy,
+    AwesomeVersionStrategyDescription,
+)
+from .typing import EnsureStrategyIterableType, EnsureStrategyType, VersionType
 from .utils.logger import LOGGER
 from .utils.regex import (
     RE_DIGIT,
     RE_MODIFIER,
-    RE_SEMVER,
     RE_SIMPLE,
     RE_VERSION,
+    compile_regex,
+    generate_full_string_regex,
     get_regex_match_group,
     is_regex_matching,
 )
@@ -23,7 +30,12 @@ class _AwesomeVersionBase(str):
     def __init__(self, string: str):
         str.__init__(string)
 
-    def __new__(cls, version: str, _: Optional[Any] = None) -> "_AwesomeVersionBase":
+    def __new__(
+        cls,
+        version: str,
+        ensure_strategy: EnsureStrategyType = None,  # pylint: disable=unused-argument
+        find_first_match: bool = False,  # pylint: disable=unused-argument
+    ) -> "_AwesomeVersionBase":
         """Create a new AwesomeVersion object."""
 
         return super().__new__(cls, version)
@@ -40,16 +52,22 @@ class AwesomeVersion(_AwesomeVersionBase):
         Match the AwesomeVersion object against spesific
         strategies when creating if. If it does not match
         AwesomeVersionStrategyException will be raised
+
+    find_first_match:
+        If True, the version given will be scanned for the first
+        match of the given ensure_strategy. Raises
+        AwesomeVersionStrategyExceptionIf it is not found
+        for any of the given strategies.
     """
 
     _version: str = ""
+    _ensure_strategy: EnsureStrategyIterableType = []
 
     def __init__(
         self,
-        version: Union[str, float, int, "AwesomeVersion"],
-        ensure_strategy: Optional[
-            Union[AwesomeVersionStrategy, List[AwesomeVersionStrategy]]
-        ] = None,
+        version: VersionType,
+        ensure_strategy: EnsureStrategyType = None,
+        find_first_match: bool = False,
     ) -> None:
         """Initialize AwesomeVersion."""
         if isinstance(version, AwesomeVersion):
@@ -60,11 +78,24 @@ class AwesomeVersion(_AwesomeVersionBase):
             self._version = self._version.strip()
 
         if ensure_strategy is not None:
-            ensure_strategy = (
+            self._ensure_strategy = ensure_strategy = (
                 ensure_strategy
-                if isinstance(ensure_strategy, list)
+                if isinstance(ensure_strategy, (list, tuple))
                 else [ensure_strategy]
             )
+            if AwesomeVersionStrategy.UNKNOWN in ensure_strategy:
+                raise AwesomeVersionStrategyException(
+                    f"Can't use {AwesomeVersionStrategy.UNKNOWN} as ensure_strategy"
+                )
+            if find_first_match:
+                for strategy in self._ensure_strategy or []:
+                    description = VERSION_STRATEGIES_DICT[strategy]
+                    match = compile_regex(description.regex_string).search(
+                        self._version
+                    )
+                    if match is not None:
+                        self._version = match.group(0)
+
             if self.strategy not in ensure_strategy:
                 raise AwesomeVersionStrategyException(
                     f"Strategy {self.strategy} does not match {ensure_strategy} for {version}"
@@ -89,39 +120,35 @@ class AwesomeVersion(_AwesomeVersionBase):
     def __str__(self) -> str:
         return str(self._version)
 
-    def __eq__(self, compareto: Union[str, float, int, object]) -> bool:
+    def __eq__(self, compareto: VersionType) -> bool:
         """Check if equals to."""
         if isinstance(compareto, (str, float, int)):
             compareto = AwesomeVersion(compareto)
         if not isinstance(compareto, AwesomeVersion):
-            raise AwesomeVersionCompare("Not a valid AwesomeVersion object")
+            raise AwesomeVersionCompareException("Not a valid AwesomeVersion object")
         return self.string == compareto.string
 
-    def __lt__(self, compareto: Union[str, float, int, object]) -> bool:
+    def __lt__(self, compareto: VersionType) -> bool:
         """Check if less than."""
         if isinstance(compareto, (str, float, int)):
             compareto = AwesomeVersion(compareto)
         if not isinstance(compareto, AwesomeVersion):
-            raise AwesomeVersionCompare("Not a valid AwesomeVersion object")
-        if (self.strategy == AwesomeVersionStrategy.UNKNOWN) or (
-            compareto.strategy == AwesomeVersionStrategy.UNKNOWN
-        ):
-            raise AwesomeVersionCompare(
-                f"Can't compare {AwesomeVersionStrategy.UNKNOWN}"
+            raise AwesomeVersionCompareException("Not a valid AwesomeVersion object")
+        if AwesomeVersionStrategy.UNKNOWN in (self.strategy, compareto.strategy):
+            raise AwesomeVersionCompareException(
+                f"Can't compare {self.strategy} and {compareto.strategy}"
             )
         return CompareHandlers(compareto, self).check()
 
-    def __gt__(self, compareto: Union[str, float, int, object]) -> bool:
+    def __gt__(self, compareto: VersionType) -> bool:
         """Check if greater than."""
         if isinstance(compareto, (str, float, int)):
             compareto = AwesomeVersion(compareto)
         if not isinstance(compareto, AwesomeVersion):
-            raise AwesomeVersionCompare("Not a valid AwesomeVersion object")
-        if (self.strategy == AwesomeVersionStrategy.UNKNOWN) or (
-            compareto.strategy == AwesomeVersionStrategy.UNKNOWN
-        ):
-            raise AwesomeVersionCompare(
-                f"Can't compare {AwesomeVersionStrategy.UNKNOWN}"
+            raise AwesomeVersionCompareException("Not a valid AwesomeVersion object")
+        if AwesomeVersionStrategy.UNKNOWN in (self.strategy, compareto.strategy):
+            raise AwesomeVersionCompareException(
+                f"Can't compare {self.strategy} and {compareto.strategy}"
             )
         return CompareHandlers(self, compareto).check()
 
@@ -221,8 +248,13 @@ class AwesomeVersion(_AwesomeVersionBase):
         if self.strategy == AwesomeVersionStrategy.SPECIALCONTAINER:
             return None
 
-        if self.strategy == AwesomeVersionStrategy.SEMVER:
-            modifier_string = get_regex_match_group(RE_SEMVER, str(self.string), 4)
+        if (
+            self.strategy_description is not None
+            and self.strategy_description.strategy == AwesomeVersionStrategy.SEMVER
+        ):
+            modifier_string = get_regex_match_group(
+                self.strategy_description.pattern, str(self.string), 4
+            )
         else:
             modifier_string = self.string.split(".")[-1]
 
@@ -234,23 +266,46 @@ class AwesomeVersion(_AwesomeVersionBase):
         if self.strategy == AwesomeVersionStrategy.SPECIALCONTAINER:
             return None
 
-        if self.strategy == AwesomeVersionStrategy.SEMVER:
-            modifier_string = get_regex_match_group(RE_SEMVER, str(self.string), 4)
+        if (
+            self.strategy_description is not None
+            and self.strategy_description.strategy == AwesomeVersionStrategy.SEMVER
+        ):
+            modifier_string = get_regex_match_group(
+                self.strategy_description.pattern, str(self.string), 4
+            )
         else:
             modifier_string = self.string.split(".")[-1]
 
         return get_regex_match_group(RE_MODIFIER, modifier_string, 3)
 
+    @property
+    def strategy_description(self) -> Optional[AwesomeVersionStrategyDescription]:
+        """Return a string representation of the strategy."""
+        if self.strategy == AwesomeVersionStrategy.UNKNOWN:
+            return None
+        return VERSION_STRATEGIES_DICT[self.strategy]
+
     @property
     def strategy(self) -> AwesomeVersionStrategy:
         """Return the version strategy."""
-        for pattern, strategy in VERSION_STRATEGIES:
-            if is_regex_matching(pattern, self.string):
-                return strategy
+        version_strategies: Dict[
+            AwesomeVersionStrategy, AwesomeVersionStrategyDescription
+        ] = {}
+
+        for strategy in self._ensure_strategy or []:
+            version_strategies[strategy] = VERSION_STRATEGIES_DICT[strategy]
+
+        for description in VERSION_STRATEGIES:
+            if description.strategy not in version_strategies:
+                version_strategies[description.strategy] = description
+
+        for description in version_strategies.values():
+            if is_regex_matching(description.pattern, self.string):
+                return description.strategy
 
         return AwesomeVersionStrategy.UNKNOWN
 
     @property
     def simple(self) -> bool:
         """Return True if the version string is simple."""
-        return is_regex_matching(RE_SIMPLE, self.string)
+        return is_regex_matching(generate_full_string_regex(RE_SIMPLE), self.string)
